/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.decorator;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.schema.api.Documentation;
import org.ow2.easywsdl.schema.api.SchemaElement;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.absItf.AbsItfAttribute;
import org.ow2.easywsdl.schema.api.absItf.AbsItfElement;
import org.ow2.easywsdl.schema.api.absItf.AbsItfImport;
import org.ow2.easywsdl.schema.api.absItf.AbsItfInclude;
import org.ow2.easywsdl.schema.api.absItf.AbsItfSchema;
import org.ow2.easywsdl.schema.api.absItf.AbsItfType;
import org.ow2.easywsdl.schema.api.extensions.NamespaceMapperImpl;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.FormChoice;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class DecoratorSchemaImpl<T extends AbsItfType, E extends AbsItfElement, A extends AbsItfAttribute, Incl extends AbsItfInclude, Impt extends AbsItfImport> implements Decorator {

	protected AbsItfSchema<T, E, A, Incl, Impt> schema;

	public DecoratorSchemaImpl(final AbsItfSchema<T, E, A, Incl, Impt> schema) {
		this.schema = schema;
	}

	public void addAttribute(final A attr) {
		this.schema.addAttribute(attr);
	}

	public void addElement(final E elmt) {
		this.schema.addElement(elmt);
	}

	public void addImport(final Impt importDef) {
		this.schema.addImport(importDef);
	}

	public void addInclude(final Incl includeDef) throws SchemaException {
		this.schema.addInclude(includeDef);
	}

	public void addType(final T absItfType) {
		this.schema.addType(absItfType);
	}

	public NamespaceMapperImpl getAllNamespaces() {
		return this.schema.getAllNamespaces();
	}

	public A getAttribute(final QName attr) {
		return this.schema.getAttribute(attr);
	}

	public List<A> getAttributes() {
		return this.schema.getAttributes();
	}

	public E getElement(final QName element) {
		return this.schema.getElement(element);
	}

	public List<E> getElements() {
		return this.schema.getElements();
	}

	public List<Impt> getImports(final String namespaceURI) {
		return this.schema.getImports();
	}

	public List<Impt> getImports() {
		return this.schema.getImports();
	}

	public List<Incl> getIncludes(final URI locationURI) throws URISyntaxException {
		return this.schema.getIncludes(locationURI);
	}

	public List<Incl> getIncludes() {
		return this.schema.getIncludes();
	}

	public String getTargetNamespace() {
		return this.schema.getTargetNamespace();
	}

	public T getType(final QName type) {
		return this.schema.getType(type);
	}

	public List<T> getTypes() {
		return this.schema.getTypes();
	}

	public Impt removeImport(final Impt importDef) {
		return this.schema.removeImport(importDef);
	}

	public Incl removeInclude(final Incl includeDef) throws SchemaException {
		return this.schema.removeInclude(includeDef);
	}

	public void setDocumentURI(final URI documentBaseURI) {
		this.schema.setDocumentURI(documentBaseURI);
	}

	public Documentation createDocumentation() {
		return this.schema.createDocumentation();
	}

	public Documentation getDocumentation() {
		return this.schema.getDocumentation();
	}

	public Map<QName, String> getOtherAttributes() throws XmlException {
		return this.schema.getOtherAttributes();
	}

	public void setDocumentation(final Documentation doc) {
		this.schema.setDocumentation(doc);
	}

	public A createAttribute() {
		return this.schema.createAttribute();
	}

	public T createComplexType() {
		return this.schema.createComplexType();
	}

	public E createElement() {
		return this.schema.createElement();
	}

	public Impt createImport() throws SchemaException {
		return this.schema.createImport();
	}

	public Incl createInclude() throws SchemaException {
		return this.schema.createInclude();
	}

	public T createSimpleType() {
		return this.schema.createSimpleType();
	}

	public void setTargetNamespace(String tns) {
		this.schema.setTargetNamespace(tns);
	}

	public FormChoice getAttributeFormDefault() {
		return this.schema.getAttributeFormDefault();
	}

	public List<String> getBlockDefault() {
		return this.schema.getBlockDefault();
	}

	public FormChoice getElementFormDefault() {
		return this.schema.getElementFormDefault();
	}

	public List<String> getFinalDefault() {
		return this.schema.getFinalDefault();
	}

	public String getLang() {
		return this.schema.getLang();
	}

	public String getVersion() {
		return this.schema.getVersion();
	}

	public URI getDocumentURI() {
		return this.schema.getDocumentURI();
	}
	
	public  List<E> findElementsInAllSchema(QName element) {
		return this.schema.findElementsInAllSchema(element);
	}
	
	public SchemaElement getInternalObject() {
		return this.schema;
	}
	
	public void setAttributeFormDefault(FormChoice form) {
		this.schema.setAttributeFormDefault(form);
	}

	public void setElementFormDefault(FormChoice form) {
		this.schema.setElementFormDefault(form);
	}
}
